local Zifnabs_Listener = {}
_G.Zifnabs_Listener = Zifnabs_Listener

local default_settings={
	["listen"]={
		["SYSTEM"]={
			[Zifnabs.LANG.LISTENER_P_CAPTION]={
				active=true,
				search={Zifnabs.LANG.LISTENER_P_ENTERED},
				say={"/p "..Zifnabs.LANG.LISTENER_WELCOME1,"/p "..Zifnabs.LANG.LISTENER_WELCOME2},
				delay={0,2500},
				},
		},
		["GUILD"]={
			[Zifnabs.LANG.LISTENER_G_CAPTION]={
				active=true,
				search={Zifnabs.LANG.LISTENER_G_ONLINE},
				say={"/g "..Zifnabs.LANG.LISTENER_WELCOME1,"/g "..Zifnabs.LANG.LISTENER_WELCOME2},
				delay={30000,50000},
				once=1,group="G1",
				},
			[Zifnabs.LANG.LISTENER_G_WB]={
				active=true,
				search={Zifnabs.LANG.LISTENER_G_ONLINE},
				say={"/g "..Zifnabs.LANG.LISTENER_WB1,"/g "..Zifnabs.LANG.LISTENER_WB2},
				delay={30000,50000},
				once=2,group="G1",
				}
		},
		["PARTY"]={
			[Zifnabs.LANG.LISTENER_LEAD_CAPTION]={
				active=true,
				search={"lead","ai"},
				alarm="ding.mp3",
			},
		},
		["YELL"]={},
		["ZONE"]={},
		["SAY"]={},
		["WHISPER"]={},
	},
}
local settings={}
local in_group=false
local time_in_group=0
local mintime_in_group=3
local done_once={}

local my_name="Zifnabs_Listener"

Zifnabs:plugin("Zifnabs_Listener",Zifnabs_Listener,true)

-- ******************************************************************************************
-- * ON EVENT
-- ******************************************************************************************
function Zifnabs_Listener:OnEvent(event,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8)
	if event=="VARIABLES_LOADED" then
		Zifnabs_Listener:reload_settings()
		local control=getglobal("Zifnabs_Listener_Control")
		control:RegisterEvent("CHAT_MSG_SYSTEM")
		control:RegisterEvent("CHAT_MSG_PARTY")
		control:RegisterEvent("CHAT_MSG_GUILD")
		control:RegisterEvent("CHAT_MSG_WHISPER")
		control:RegisterEvent("CHAT_MSG_ZONE")
		control:RegisterEvent("CHAT_MSG_YELL")
		control:RegisterEvent("CHAT_MSG_CHANNEL")
		control:RegisterEvent("CHAT_MSG_SAY")
		control:RegisterEvent("PARTY_MEMBER_CHANGED")
		Zifnabs:dofile("Zifnabs_Config_Listener")	
	elseif event=="SAVE_VARIABLES" then 
		Zifnabs:save_settings(default_settings,settings)
	elseif event=="PARTY_MEMBER_CHANGED" then
		in_group=false
		for p=1,32,1 do
			if UnitExists("raid"..p) then
				in_group=true
				break
			end
		end
	elseif event=="CHAT_MSG_SYSTEM" and type(arg1)=="string" then Zifnabs_Listener:scan_message("SYSTEM",arg1,arg4)
	elseif event=="CHAT_MSG_PARTY" then Zifnabs_Listener:scan_message("PARTY",arg1,arg4)
	elseif event=="CHAT_MSG_GUILD" and type(arg1)=="string" then Zifnabs_Listener:scan_message("GUILD",arg1,arg4)
	elseif event=="CHAT_MSG_WHISPER" and type(arg1)=="string" then Zifnabs_Listener:scan_message("WHISPER",arg1,arg4)
	elseif event=="CHAT_MSG_ZONE" and type(arg1)=="string" then Zifnabs_Listener:scan_message("ZONE",arg1,arg4)
	elseif event=="CHAT_MSG_YELL" and type(arg1)=="string" then Zifnabs_Listener:scan_message("YELL",arg1,arg4)
	elseif event=="CHAT_MSG_CHANNEL" and type(arg1)=="string" then Zifnabs_Listener:scan_message("CHANNEL",arg1,arg4)
	elseif event=="CHAT_MSG_SAY" and type(arg1)=="string" then Zifnabs_Listener:scan_message("SAY",arg1,arg4)
	else Zifnabs:echo("Listener uncaught event: "..event.." a1='"..(arg1 or "nil").."' tig="..time_in_group)		
	end
end

function Zifnabs_Listener:OnUpdate(elapsed,last,updates,since_load)
	if in_group then time_in_group=time_in_group+elapsed else time_in_group=0 end
end

function Zifnabs_Listener:reload_settings(reset)
	if reset=="reset" then Zifnabs_Settings.listen=nil end
	settings=Zifnabs:init_settings(default_settings)
end

function Zifnabs_Listener:scan_message(channel,message,issuer)
	if type(settings.listen[channel])=="table" then
		local msg=" "..string.gsub(string.gsub(string.lower(message),"%p"," "),"%c","").." "
		local action=nil
		local target=Zifnabs_Listener:msg_issuer(message,issuer)
		if target~=UnitName("player") then
			for caption,entry in pairs(settings.listen[channel]) do
				if entry.active==true then
					if entry.once==nil
					or (entry.once~=2 and done_once[channel.."/"..(entry.group or "default").."/"..target]==nil)
					or (entry.once~=1 and done_once[channel.."/"..(entry.group or "default").."/"..target]~=nil) then
						for s=1,#entry.search,1 do
							if string.find(msg," "..entry.search[s].." ")~=nil then
								if entry.search[s]~=Zifnabs.LANG.LISTENER_P_ENTERED or time_in_group>mintime_in_group then 
									action=entry
									action.found=entry.search[s]
									break
								end
							end
						end
					end
					if action~=nil then break end
				end
			end
		end
		if action~=nil then
			if action.once==1 or action.once==2 then done_once[channel.."/"..(action.group or "default").."/"..target]=true end
			if type(action.alarm)=="string" then 
				PlaySoundByPath("Interface/Addons/Zifnabs/media/"..action.alarm); 
				Zifnabs:echo("'"..action.found.."' "..Zifnabs.LANG.LISTENER_FOUND_IN.." "..(Zifnabs.LANG.LISTENER_CHANNEL[channel] or "???"))
			end
			if type(action.say)=="table" then
				if #action.say>0 then
					local delay=0
					if type(action.delay)=="table" then delay=action.delay[1]+math.random(action.delay[2]-action.delay[1]) end
					local text=action.say[math.random(#action.say)]
					if string.find(text,"%t")==nil or target~="" then 
						Zifnabs:timer("add",0,delay,"chat",string.gsub(text,"%%t",target)) 
					end
				end
			end
		end
	end
end

function Zifnabs_Listener:msg_issuer(message,issuer)
	local target=issuer or ""
	if target=="" and message~=nil and message~="" then 
		if string.lower(string.sub(message,1,2))=="|c" then 
			message=string.sub(message,11) 
			if string.find(message,"|r")~=nil then message=string.sub(message,1,string.find(message,"|r")-1) end
		end
		while string.sub(message,1,1)==" " do message=string.sub(message,2) end
		local stop=string.find(message.." "," ")
		target=string.sub(message,1,stop)
		if string.sub(target,1,1)=="[" and string.find(arg1,"]")~=nil then 
			target=string.sub(target,2,string.find(target,"]")-1)
		end
	end
	return target
end
function Zifnabs_Listener:once()
	for entry,boolean in pairs(done_once) do Zifnabs:echo(entry) end
end